

// ------------------------------------------------------------------------------
//
// LudumDare.com 48hr compo, July 2002
//
// Dave Reed (Bluescreen)'s entry
//
// The game itself
//
// ------------------------------------------------------------------------------


#include <windows.h>
#include <gl\gl.h>
#include <stdio.h>
#include <string.h>

#include "winmain.h"
#include "game.h"
#include "sprites.h"
#include "flames.h"
#include "math.h"
#include "maps.h"


GAMEINFO gameInfo;
SPRITE spBGTile;
SPRITE spMainTiles[16];
SPRITE spBlob[10];

char map[32][32];
float damage[32][32];
float burning[32][32];


// ------------------------------------------------------------------------------
// Init stuff
// ------------------------------------------------------------------------------

void GAME_initLevel(int level)
{
	int x,y,lev;
	gameInfo.worldAngle=0;
	

	// Load map, rot/flip to sensible coords system
	// --------------------------------
	for(y=0;y<32;y++)
		for(x=0;x<32;x++)
		{
			if (gameInfo.level>8) gameInfo.level=1;
			lev=gameInfo.level;

			if (lev==1) map[x][y]=map1[31-y][x];
			if (lev==2) map[x][y]=map2[31-y][x];
			if (lev==3) map[x][y]=map3[31-y][x];
			if (lev==4) map[x][y]=map4[31-y][x];
			if (lev==5) map[x][y]=map5[31-y][x];
			if (lev==6) map[x][y]=map6[31-y][x];
			if (lev==7) map[x][y]=map7[31-y][x];
			if (lev==8) map[x][y]=map8[31-y][x];

			
			
			damage[x][y]=0;
			burning[x][y]=0;
		}

	
	gameInfo.bOB=FALSE;

	gameInfo.woodTiles=0;
	gameInfo.damaged=0;
	gameInfo.destroyed=0;
	gameInfo.damage=0;
	gameInfo.bToasted=FALSE;

	FLAMES_init();

	// Find entities/special tiles
	// --------------------------------
	for(y=0;y<32;y++)
		for(x=0;x<32;x++)
		{

			if (map[x][y]==3)
			{
				map[x][y]=1;
				FLAME_add(x,y);
			}

			// Start pos
			if (map[x][y]==4)
			{
				map[x][y]=0;
				gameInfo.vPos.x=x;
				gameInfo.vPos.y=y;
				gameInfo.vPos.z=-2.0f;			
			}			

			if (map[x][y]==1) gameInfo.woodTiles++;
		}


	gameInfo.vVel.x=gameInfo.vVel.z=gameInfo.vVel.y=0;
	gameInfo.zoom=-3.0f;
	gameInfo.targetZoom=-30.0f;
	gameInfo.delayTimer=180;
	
	if (gameInfo.state!=GAME_TITLESCREEN) SFX_play(SFX_READY);

	gameInfo.state=GAME_GETREADY;
}

void GAME_init()
{
	int x,y;

	SFX_load(SFX_SLIME,"assets/slime.wav");
	SFX_load(SFX_READY,"assets/ready.wav");
	SFX_load(SFX_GO,"assets/go.wav");
	SFX_load(SFX_IGNITE,"assets/ignite.wav");
	SFX_load(SFX_POP,"assets/pop.wav");
	SFX_load(SFX_DEAD ,"assets/bollocks.wav");
	SFX_load(SFX_HISS,"assets/extinguish.wav");

	SFX_load(SFX_DRIP,"assets/drip1.wav");
	SFX_load(SFX_DRIP+1,"assets/drip2.wav");
	SFX_load(SFX_DRIP+2,"assets/drip3.wav");



	gameInfo.level=1;
	gameInfo.state=GAME_TITLESCREEN;
	GAME_initLevel(gameInfo.level);
	gameInfo.state=GAME_TITLESCREEN;

}


// ------------------------------------------------------------------------------
// InitTextures - gets called again after a fullscreen-windowed switch
// ------------------------------------------------------------------------------

void GAME_initTextures()
{		
	GL_loadTexture(100,"assets\\bgtile.png");    // background tile

	
	
	GL_loadTexture(0,"assets\\tile.png");			
	GL_loadTexture(1,"assets\\metal.png");			
	GL_loadTexture(2,"assets\\slime.png");			
	GL_loadTexture(10,"assets\\tile_slimed.png");			
	GL_loadTexture(11,"assets\\metal_slimed.png");			

	GL_loadTexture(126,"assets\\blob2.bmp");			
	GL_loadTexture(127,"assets\\blob.bmp");			


	SPRITE_init(&spBlob[1],1.0f,1.0f,0.5f,0.5f,126,32,32,0,0,32,32);
	SPRITE_init(&spBlob[0],1.0f,1.0f,0.5f,0.4f,127,32,32,0,0,32,32);

	SPRITE_init(&spBGTile,1.0f,1.0f,0.5f,0.5f,100,64,64,0,0,64,64);
	SPRITE_init(&spMainTiles[0],1.0f,1.0f,0.5f,0.5f,0,64,64,0,0,64,64);
	SPRITE_init(&spMainTiles[1],1.0f,1.0f,0.5f,0.5f,1,64,64,0,0,64,64);
	SPRITE_init(&spMainTiles[2],1.0f,1.0f,0.5f,0.5f,2,64,64,0,0,64,64);
	SPRITE_init(&spMainTiles[10],1.0f,1.0f,0.5f,0.5f,10,64,64,0,0,64,64);
	SPRITE_init(&spMainTiles[11],1.0f,1.0f,0.5f,0.5f,11,64,64,0,0,64,64);

	FLAMES_initTextures();

	
}


// ------------------------------------------------------------------------------
// Deinit stuff
// ------------------------------------------------------------------------------

void GAME_deinit()
{
	SFX_free(SFX_SLIME);
	SFX_free(SFX_READY);
	SFX_free(SFX_GO);
	SFX_free(SFX_IGNITE);
	SFX_free(SFX_POP);
	SFX_free(SFX_DEAD);
	SFX_free(SFX_HISS);
	SFX_free(SFX_DRIP);
	SFX_free(SFX_DRIP+1);
	SFX_free(SFX_DRIP+2);
}


// ------------------------------------------------------------------------------
// Update the game logic
// ------------------------------------------------------------------------------

void slime(int x,int y)
{
	if (x<0 || y<0 || x>31 || y>31) return;

	if (burning[x][y]<=0)
	{
		if (map[x][y]==1) {map[x][y]=11; SFX_play(SFX_SLIME);}
		if (map[x][y]==2) {map[x][y]=12; SFX_play(SFX_SLIME);}		
	}
}

#define TOAST 0.25f

int collide()
{
	int x,y,r=0;
	

	x=(gameInfo.vPos.x+0.4f);
	y=(gameInfo.vPos.y+0.6f);

	if (x<0 || y<0 || x>31 || y>31) return 0;

	if (map[x][y]!=0)
	{
		slime(x,y);
		r|=map[x][y];

		if (burning[x][y]>TOAST) gameInfo.bToasted=TRUE;					
	}

	x=(gameInfo.vPos.x+0.4f);
	y=(gameInfo.vPos.y+0.4f);
	if (map[x][y]!=0)
	{
		slime(x,y);
		r|=map[x][y];

		if (burning[x][y]>TOAST) gameInfo.bToasted=TRUE;					
	}

	x=(gameInfo.vPos.x+0.6f);
	y=(gameInfo.vPos.y+0.6f);
	if (map[x][y]!=0)
	{
		slime(x,y);
		r|=map[x][y];

		if (burning[x][y]>TOAST) gameInfo.bToasted=TRUE;					
	}

	x=(gameInfo.vPos.x+0.4f);
	y=(gameInfo.vPos.y+0.4f);
	if (map[x][y]!=0)
	{
		slime(x,y);
		r|=map[x][y];

		if (burning[x][y]>TOAST) gameInfo.bToasted=TRUE;					
	}


	return r;

}

void GAME_update()
{

	VECTOR *pos=&gameInfo.vPos;
	VECTOR *vel=&gameInfo.vVel;
	VECTOR *grav=&gameInfo.vGrav;


	if (gameInfo.state==GAME_RUNNING)
	{

		// Update the player pos
		pos->x+=vel->x;	

		if (collide())
		{
			gameInfo.onGround=3;
			pos->x-=vel->x;	
			vel->x=-vel->x/4.0f;	
		}

		pos->y+=vel->y;

		if (collide())
		{
			gameInfo.onGround=3;
			pos->y-=vel->y;	
			vel->y=-vel->y/4.0f;	
		}

		// Accelerate it
		vel->x+=grav->x;
		vel->y+=grav->y;

	}

}



// ------------------------------------------------------------------------------
// Draw stuff
// ------------------------------------------------------------------------------

void GAME_render() 
{
	VECTOR vPos,vBollocks;
	static float ang=0;	
	static float scroll,px=16,py=16;
	float spd,rv2,ox,oy;
	static float rv=0,ticks=0;
	int x,y,z,xv,yv,zv;


	ticks++;
	
	if (gameInfo.state==GAME_RUNNING)
	{
		if (g_keyState[VK_LEFT])  gameInfo.worldAngle+=2.6f;
		if (g_keyState[VK_RIGHT]) gameInfo.worldAngle-=2.6f;
	}


	if (gameInfo.worldAngle<0.0f)   gameInfo.worldAngle+=360.0f;
	if (gameInfo.worldAngle>360.0f) gameInfo.worldAngle-=360.0f;
	

	if (gameInfo.onGround)
	{		
		rv=0;

		if (gameInfo.worldAngle<90 || gameInfo.worldAngle>270) 		
			rv-=(gameInfo.vVel.x*80.0f);		
		else		
			rv+=(gameInfo.vVel.x*80.0f);		


		if (gameInfo.worldAngle<180)
			rv+=(gameInfo.vVel.y*80.0f);
		else
			rv-=(gameInfo.vVel.y*80.0f);
				
//		if (spBlob[0].ox>0.1f) spBlob[0].ox=0.1f;
//		if (spBlob[0].ox<-0.1f) spBlob[0].ox=-0.1f;


		gameInfo.onGround--;
		
		
	}

	gameInfo.headAng=-gameInfo.worldAngle;
	scroll+=rv;

	rv2=rv*6.0f;
	if (rv2>25.0f) rv2=25;
	if (rv2<-25.0f) rv2=-25;
	spBlob[0].ox=(spBlob[0].ox*0.95f)+ ((0.5f+(rv2/100.0f))*0.05f); 

		
	
	gameInfo.zoom=(gameInfo.zoom*0.99f)+(gameInfo.targetZoom*0.01f);

	if (gameInfo.state==GAME_RUNNING)
	{
		spd=MATH_normalizeVector(&gameInfo.vVel,&vBollocks);
		gameInfo.targetZoom=-spd*220.0f;
		if (gameInfo.targetZoom<-40) gameInfo.targetZoom=-40;
		if (gameInfo.targetZoom>-7) gameInfo.targetZoom=-7;
	}		
	else
		gameInfo.targetZoom=-50;
	
	

	// Update gravity vector
	gameInfo.vGrav.x=-sin(gameInfo.worldAngle*DEG_TO_RAD)/256.0f;
	gameInfo.vGrav.y=-cos(gameInfo.worldAngle*DEG_TO_RAD)/256.0f;


	// Clear Screen And Depth Buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	

	// Position camera
	glLoadIdentity();			
	glRotatef(gameInfo.worldAngle,0,0,1.0f);
	glTranslatef(-gameInfo.vPos.x,-gameInfo.vPos.y,gameInfo.zoom);	

	// Enable texturing, disable filtering
	glDisable(GL_POLYGON_SMOOTH);	
	glEnable(GL_TEXTURE_2D);
	
	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	
	//glDisable(GL_DEPTH_TEST);

	// Light it
	
	//glColor3f(0.25f,0.25f,0.25f);


	for(x=-10; x<14;x++)
		for(y=-10; y<14;y++)
		{
			glColor3f(0.35f,0.35f,0.35f);
			vPos.x=x<<4;
			vPos.y=y<<4;			
			vPos.z=-50.0f;
			SPRITE_blit(&spBGTile,&vPos,0,16.0f);	
		}
	

	vPos.z=-3.0f;	
	glColor3f(0.75f,0.75f,0.75f);

	glBlendFunc(GL_ONE,GL_ONE); //SRC_COLOR,GL_ONE);	
	glEnable(GL_BLEND);		

	
	SPRITE_blit(&spBlob[1],&gameInfo.vPos,scroll,0.75f); 
	
	//if (gameInfo.onGround)
	//	SPRITE_blit(&spBlob[0],&gameInfo.vPos,,0.75f);	
	//else
		SPRITE_blit(&spBlob[0],&gameInfo.vPos,gameInfo.headAng,0.75f);	

	glDisable(GL_BLEND);
	

	


	
	glColor3f(0.75f,0.75f,0.75f);
	for(x=0; x<32;x++)
		for(y=0; y<32;y++)
		{
			int t=map[x][y];
						
			if (t)
			{
				vPos.x=x;
				vPos.y=y;
				
				

				if (t>9)
				{
					float i;
					ox=(sin((ticks+((x+y)*4))/7.0f))/15.0f;
					oy=(sin((ticks+((x+y)*4))/5.0f))/15.0f;

					vPos.x+=ox;
					vPos.y+=oy;
										
					i=(1.0f)-(damage[x][y]/1.3f);
					SPRITE_blitTile(&spMainTiles[t-1]-10,&vPos,damage[x][y]*15.0f,0.8f,2.0f,1.0f,i,i,i);					

					glEnable(GL_BLEND);
					glDisable(GL_DEPTH_TEST);
					glDisable(GL_CULL_FACE);
					i=0.75f+(ox*3.0f);
					SPRITE_blitTile(&spMainTiles[2],&vPos,ox*90.0f,1.00f,2.2f,0.8f,i,i,i);
					glEnable(GL_DEPTH_TEST);
					glEnable(GL_CULL_FACE);
					glDisable(GL_BLEND);

					if ((rand()&31)>30) FLAME_addSlime(x,y);
				}
				else
				{
					float i=(1.0f)-(damage[x][y]/1.3f);
					SPRITE_blitTile(&spMainTiles[t-1],&vPos,damage[x][y]*15.0f,0.8f,2.0f,1.0f,i,i,i);
				}

				//else
					


				
			}		
		}


		

	if (gameInfo.state!=GAME_TITLESCREEN)
	{
		FLAMES_update();
	}


	if (gameInfo.state==GAME_GETREADY)
	{
		char tmp[80];
		sprintf(tmp," Level %d",gameInfo.level);
		GL_print(tmp,0.47f,0.36f);
		GL_print("Get Ready!",0.455f,0.40f);

		gameInfo.delayTimer--;

		if (gameInfo.delayTimer<=0)
		{
			gameInfo.state=GAME_RUNNING;
			SFX_play(SFX_GO);
		}
	}
	


	// dead - out of bounds
	if (( (gameInfo.vPos.x<-20.0f) || (gameInfo.vPos.x>52.0f)
		||(gameInfo.vPos.y<-20.0f) || (gameInfo.vPos.y<-52.0f))
		 && gameInfo.state!=GAME_ENDOFPLAY)
	{
		gameInfo.bOB=TRUE;
		gameInfo.state=GAME_ENDOFPLAY;
		SFX_play(SFX_DEAD);
	}

	// dead - too much fire damage
	if (gameInfo.damage>50.0f && gameInfo.state!=GAME_ENDOFPLAY)
	{
		gameInfo.state=GAME_ENDOFPLAY;
		SFX_play(SFX_DEAD);
	}


	// dead - hit flames

	if (gameInfo.bToasted  && gameInfo.state!=GAME_ENDOFPLAY)
	{
		gameInfo.state=GAME_ENDOFPLAY;
		SFX_play(SFX_DEAD);
	}


	if (gameInfo.state==GAME_ENDOFPLAY)
	{
		
		gameInfo.delayTimer++;

		if (gameInfo.flameCount==0)		
		{
			GL_print("Fire Extinguished!",0.418f,0.36f);
			GL_print("Well Done!",0.455f,0.40f);

			
			if (gameInfo.delayTimer>180)
			{
				gameInfo.level++;
				GAME_initLevel(gameInfo.level);
			}
		}
		else
		{
			if (gameInfo.damage>50.0f)
			{
				GL_print("Too Much Damage!",0.418f,0.36f);

				if (gameInfo.lives>0)
				{									
					GL_print("Try Again!",0.455f,0.40f);
					if (gameInfo.delayTimer>180) GAME_initLevel(gameInfo.level);
				}
				else
				{
					gameInfo.level=1;
					GL_print("Game Over!",0.455f,0.40f);
					
					if (gameInfo.delayTimer>180)
					{
						gameInfo.state=GAME_TITLESCREEN;
						GAME_initLevel(gameInfo.level);
						gameInfo.state=GAME_TITLESCREEN;
					}					
				}
				
				
			}
			else
			{
				if (gameInfo.bToasted)
					GL_print("Toasted!",0.463f,0.36f);
				else
					GL_print("Out Of Bounds!",0.418f,0.36f);
				
				if (gameInfo.lives>0)
				{
					GL_print("Try Again!",0.455f,0.40f);
					if (gameInfo.delayTimer>180)
					{
						gameInfo.lives--;
						GAME_initLevel(gameInfo.level);
					}
				}
				else
				{
					gameInfo.level=1;
					GL_print("Game Over!",0.455f,0.40f);
					
					if (gameInfo.delayTimer>180)
					{
						gameInfo.state=GAME_TITLESCREEN;
						GAME_initLevel(gameInfo.level);
						gameInfo.state=GAME_TITLESCREEN;
					}
					
				}

				
			}
		}
		
		if (gameInfo.bOB)
			gameInfo.targetZoom=-4.0f;		
		else
			gameInfo.targetZoom=-50.0f;		
	}
	


	if (gameInfo.state!=GAME_TITLESCREEN)
	{
		GL_print("Damage:",0,0.01f);
		GL_print("Level: ",0,0.05f);
		GL_print("Lives: ",0,0.09f);

		{
			char tmp[10];
			float damage;

			
			damage=(gameInfo.destroyed+gameInfo.damaged);
			damage/=(((float)(gameInfo.woodTiles)));
			damage*=50.0f;

			gameInfo.damage=damage;

			sprintf(tmp,"%04.1fn",damage);
			tmp[4]='%';
			GL_print(tmp,0.3f,0.01f);
			
			sprintf(tmp,"%d",gameInfo.level);
			GL_print(tmp,0.3f,0.05f);

			sprintf(tmp,"%d",gameInfo.lives);
			GL_print(tmp,0.3f,0.09f);
		}
	}


	if (gameInfo.state==GAME_TITLESCREEN)
	{
		GAME_renderTitScreen();
	}
}





void GAME_renderTitScreen(void)
{
	GL_print("'The Flame Game' v0.1",0.1f,0.01f);

	GL_print("By Dave Reed (bluescreen)",0.1f,0.10f);
	GL_print("Dave_Reed@btopenworld.com",0.1f,0.14f);

	GL_print("Controls: <- and -> to rotate the world",0.1f,0.24f);
	GL_print("Green slime puts out fire! Protect the level and blob!",0.1f,0.28f);

	GL_print("Keep fire damage below 50% to continue",0.1f,0.36f);
	GL_print("Fire spreads fastest upwards! Metal blocks don't burn!",0.1f,0.40f);

	GL_print("Written in under 48hrs for the www.ludumdare.com compo!",0.1f,0.48f);
	GL_print("Hello to all from #ludumdare...",0.1f,0.52f);

	GL_print("Uses MSVC++, OpenGL (NeHe init code), DevIL, and FMOD",0.1f,0.56f);

	GL_print("Press <space> to play!",0.1f,0.65f);

	gameInfo.targetZoom=-0.0f;

	gameInfo.worldAngle+=1.0f;
	
	if (g_keyState[VK_SPACE])
	{
		gameInfo.lives=3;
		gameInfo.level=1;
		GAME_initLevel(gameInfo.level);
	}
}
